#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <inttypes.h>
#include <math.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <zlib.h>
#include <omp.h>	// OMP multithreading
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include "hashtest.h"
#include "util.h"
//static struct nlist *hashtable[HASHSIZE];

#define HASHTABLELENGTH 4093
#define HASHTABLEWIDTH	10

static struct nlist *hashtable;
static struct nlist *conflict_table;
static int conflict_table_len;
static int conflict_table_size;
#define IN_HASHTABLE	1
#define IN_CONFLICTTABLE	2
#define NOT_FOUND_IN_HASH	3
#define NOT_FOUND_IN_CONFLICT	4

void
print_hashtable(){
        int i,j;
        struct nlist *p;
        for(i=0;i<HASHTABLELENGTH*HASHTABLEWIDTH;i++){
                p = &hashtable[i];
                if (p->value){
                        printf("%i\t%i\t%i\n",i,p->value,p->multi_value);
                }
        }
}

int
hash_setup(int dblen){
	conflict_table_size=dblen;
	conflict_table_len=0;
	hashtable=(nlist*)xcalloc(HASHTABLELENGTH*HASHTABLEWIDTH*sizeof(struct nlist));
	conflict_table=(nlist*)xcalloc(conflict_table_size*sizeof(struct nlist));

	return 1;
}
int
arraycmp(int8_t *a1, int8_t *a2, int len){

	int i;

	for(i=0;i<len;i++){

		if(a1[i]!=a2[i])return 1;
	}

	return 0;

}
int8_t*
arraydup(int8_t *a, int len){
	int8_t *a1=(int8_t*)xmalloc(len*sizeof(int8_t));
	if(a1==NULL){
		fprintf(stderr,"arraydup failed.\n");
		abort();
	}
	memcpy(a1,a,len*sizeof(int8_t));

	return a1;
}
int
hash(int8_t *s, int len)
{
    uint64_t hashval;

    int i;
    for (i=0,hashval = 0; i<len; i++)
        hashval = s[i] + 31 * hashval;

    return (hashval % HASHTABLELENGTH)*HASHTABLEWIDTH;
}
int
hashtable_search(int8_t *s, int len, int *hashval, int *table_type)
{
    int i;
    //int type;
    //bool found=false;
    *hashval=hash(s,len);

    for (i=*hashval; i<*hashval+HASHTABLEWIDTH; i++){
	if(hashtable[i].value == 0){
		*table_type=NOT_FOUND_IN_HASH;
		return i;
	}
    	if (arraycmp(s,hashtable[i].keys,len) == 0){
    		*table_type= IN_HASHTABLE;
		return i;
        }
    }
    for(i=0;i<conflict_table_len;i++){
	if(conflict_table[i].value == 0){
		//*table_type=NOT_FOUND_IN_CONFLICT;
		//return i;
		break;
	}
    	if (arraycmp(s, conflict_table[i].keys,len) == 0){
    		*table_type= IN_CONFLICTTABLE;
		return i;
	}
    }
    *table_type=NOT_FOUND_IN_CONFLICT;
    return i;
}
int
hashtable_insert(int8_t *keys, int value,int len)
{
    struct nlist *np;
    int hashval;
    int type;
    int idx;
    idx=hashtable_search(keys,len,&hashval,&type);
    switch(type){
    case NOT_FOUND_IN_HASH:
    	hashtable[idx].keys=keys;
    	hashtable[idx].value=value;
	break;
    case IN_HASHTABLE:
    	hashtable[idx].multi_value=true;
	break;
    case IN_CONFLICTTABLE:
    	conflict_table[idx].multi_value=true;
	break;
    case NOT_FOUND_IN_CONFLICT:
	if(conflict_table_len==conflict_table_size-1){
		fprintf(stderr,"conflict table full, skip.\n");
		return 0;
	}
	conflict_table[conflict_table_len].keys=keys;
        conflict_table[conflict_table_len].value=value;
	conflict_table_len++;
	break;
    default:
	fprintf(stderr,"unknown type:%i.\n",type);
	abort();
    }

    return 1;
}

struct nlist*
hashtable_getvalue(int8_t *keys, int len)
{
	int hashval,idx, type;
	idx=hashtable_search(keys,len,&hashval, &type);
	switch(type){
	case NOT_FOUND_IN_HASH:
	case NOT_FOUND_IN_CONFLICT:
		return NULL;
	case IN_HASHTABLE:
	        return &hashtable[idx];
	case IN_CONFLICTTABLE:
	        return &conflict_table[idx];
	default:
		fprintf(stderr,"unknown type:%i.\n",type);
        	abort();
	}

}
void
hashtable_clean(){
	memset(hashtable,0,HASHTABLELENGTH*HASHTABLEWIDTH*sizeof(struct nlist));
	memset(conflict_table,0,conflict_table_len*sizeof(struct nlist));
	conflict_table_len=0;
}

