#ifndef _GMAPPER_H
#define _GMAPPER_H

#include <stdlib.h>

/* pair mode definitions */
#define PAIR_NONE	0
#define PAIR_OPP_IN	1
#define PAIR_OPP_OUT	2
#define PAIR_COL_FW	3
#define PAIR_COL_BW	4

static char const * const pair_mode_string[5] =
  { "none", "opposing strands; inwards", "opposing strands; outwards",
    "same strand; second is forward", "same strand; second is backward" };

static bool const pair_reverse[5][2] =
  { { 0, 0 }, // PAIR_NONE
    { 0, 0 }, // PAIR_OPP_IN
    { 1, 1 }, // PAIR_OPP_OUT
    { 0, 1 }, // PAIR_COL_FW
    { 1, 0 }  // PAIR_COL_BW
  };

/* defaults */
#define DEF_NUM_THREADS 1
#define DEF_CHUNK_SIZE 1000

#define DEF_HASH_FILTER_CALLS	true
#define DEF_GAPLESS_SW		false
#define DEF_LIST_CUTOFF		4294967295u // 2^32 - 1

#define DEF_PAIR_MODE		PAIR_NONE
#define DEF_MIN_INSERT_SIZE	50
#define DEF_MAX_INSERT_SIZE	2000

#define DEF_WINDOW_LEN		140.0
#define DEF_WINDOW_OVERLAP	20.0
#define DEF_NUM_MATCHES		2
#define DEF_NUM_OUTPUTS		10
#define DEF_ANCHOR_WIDTH	8	/* width around anchors in full SW */

/* SW Scores */
#define DEF_MATCH_VALUE		10
#define DEF_MISMATCH_VALUE	-15
#define DEF_A_GAP_OPEN		-40
#define DEF_B_GAP_OPEN		DEF_A_GAP_OPEN
#define DEF_A_GAP_EXTEND	-7
#define DEF_B_GAP_EXTEND	DEF_A_GAP_EXTEND
#define DEF_XOVER_PENALTY	-14	/* CS only */

/* Score Thresholds */
#define DEF_WINDOW_GEN_THRESHOLD	55.0	/* Min required to generate match window */
#define DEF_SW_VECT_THRESHOLD	60.0	/* == DEF_SW_FULL_THRESHOLD in lspace */
#define DEF_SW_FULL_THRESHOLD	68.0	/* read_length x match_value x .68 */


/*
 * The maximum seed weight (maximum number of 1's in the seed) sets an
 * upper limit on our lookup table allocation size. The memory usage of
 * rmapper corresponds strongly to 4^MAX_SEED_WEIGHT * (sizeof(void *) +
 * sizeof(uint32_t)). At 16, this is 32GB on 32-bit and 48GB on 64-bit
 * architectures.
 */
#define MAX_SEED_WEIGHT		14
#define MAX_SEED_SPAN		64

/*
 * For larger seeds we'll just use a hash table. Presently, we're restricted to
 * 128 bytes in kmer_to_mapidx, but it's trivially extended.
 */
#define MAX_HASH_SEED_WEIGHT	64
#define MAX_HASH_SEED_SPAN	64
#define HASH_TABLE_POWER	12	/* 4^HASH_POWER entries in table */


#define MAX_ANCHOR_LIST		10000000

/*Split Mapping*/
#define CUR_1_THRESH 10
#define CUR_2_THRESH 3
#define CUR_3_THRESH 3
#define CUR_4_THRESH 3
#define MIN_INS_MATCH_LEN 25
#define SPLIT_PENALTY_INS -50
#define SPLIT_PENALTY_DEL_LARGE 1
#define SPLIT_PENALTY_DEL_DOUBLE -49

/*
 * If window_len, sw_vect_threshold, sw_full_threshold are absolute values,
 * we'll set them negative to distinguish.
 */



struct sam_entry{
	char rname[100];
	int strand;//flag
	char genome_name[100];
	uint32_t pos;
	int mapq;
	char cigar[100];
	char mate_ref_name[100];
	uint32_t mpos;
	int isize;
	char seq[1024];
	char qval[1024];
	char mseq[1024];
	int difficulty;
	int mismatch;
	int type;
	char chr1[100];
	char chr2[100];
};
struct cnv_breakpoint{
	int bp1;
	char chr1[100];
	char chr2[100];
	int bp2;
};
#endif

