#/usr/bin/env bash
set -eu
set -o pipefail

prism_sam="_none"
loci="_none"
sort_mem="1g"
usage()
{
        echo "usage:"
        echo "  -i PRiSM output sam file."
	echo "  -l SV location file (e.g. FILTERED_LOCI)."
        echo "  -S memory for linux sort (default: 1g)."
        exit 0
}

#export PRISM_PATH

if test ! -d ${PRISM_PATH} || [ -z ${PRISM_PATH} ];then
        echo  "PRISM_PATH(${PRISM_PATH}) is invalid."
        exit 0;
fi
if test ! -e "${PRISM_PATH}/bin/smapper";then
        echo "PRiSM bin files lost."
        exit 0
fi

while getopts "i:l:S:" options;do
        case $options in
                i)      prism_sam=$OPTARG;echo -e "PRiSM sam:\t${prism_sam}";;
                l)      loci=$OPTARG;echo -e "SV loci file:\t${loci}";;
		S)	sort_mem=$OPTARG;;
                \?)     usage;;
        esac
done

if test ! -e ${prism_sam};then
        echo "input file is invalid:$prism_sam."
        usage
        exit 0
fi
if test ! -e ${loci};then
        echo "input file is invalid:$loci."
        usage
        exit 0
fi
echo "PLEASE NOTE:The output sam file from this script is only for screening the reads in a genome browser. All the reads which are mapped to the same
	location and contain the same SV will be screened out (Since only one of these reads is used in the SV calling step, users will probably see
	more reads supporting a SV here)."

output_path=$(dirname "${prism_sam}")
sort -S ${sort_mem} -k4,4n ${prism_sam} -o ${prism_sam}
${PRISM_PATH}/bin/smapper screenout ${prism_sam} ${loci}

